part of forestvpn_api.api;



class WireguardApi {
  final ApiClient apiClient;

  WireguardApi([ApiClient apiClient]) : apiClient = apiClient ?? defaultApiClient;

  /// Create new WireGuard profile with HTTP info returned
  ///
  /// 
  Future<Response> createProfileWithHttpInfo({ WireGuardProfileCreateRequest request }) async {
    Object postBody = request;

    // verify required params are set

    // create path and map variables
    String path = "/wg/profiles/".replaceAll("{format}","json");

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};

    List<String> contentTypes = ["application/json"];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'POST',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Create new WireGuard profile
  ///
  /// 
  Future<WireguardProfile> createProfile({ WireGuardProfileCreateRequest request }) async {
    Response response = await createProfileWithHttpInfo( request: request );
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
      return apiClient.deserialize(_decodeBodyBytes(response), 'WireguardProfile') as WireguardProfile;
    } else {
      return null;
    }
  }

  /// Delete WireGuarg profile with HTTP info returned
  ///
  /// 
  Future deleteProfileWithHttpInfo(String profileId) async {
    Object postBody;

    // verify required params are set
    if(profileId == null) {
     throw ApiException(400, "Missing required param: profileId");
    }

    // create path and map variables
    String path = "/wg/profiles/{profileId}/".replaceAll("{format}","json").replaceAll("{" + "profileId" + "}", profileId.toString());

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};

    List<String> contentTypes = [];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'DELETE',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Delete WireGuarg profile
  ///
  /// 
  Future deleteProfile(String profileId) async {
    Response response = await deleteProfileWithHttpInfo(profileId);
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
    } else {
      return;
    }
  }

  /// Retrieve Wireguard peers with HTTP info returned
  ///
  /// 
  Future<Response> getPeersWithHttpInfo({ int timestamp }) async {
    Object postBody;

    // verify required params are set

    // create path and map variables
    String path = "/wg/peers/".replaceAll("{format}","json");

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};
    if(timestamp != null) {
      queryParams.addAll(_convertParametersForCollectionFormat("", "timestamp", timestamp));
    }

    List<String> contentTypes = [];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'GET',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Retrieve Wireguard peers
  ///
  /// 
  Future<List<WireguardPeer>> getPeers({ int timestamp }) async {
    Response response = await getPeersWithHttpInfo( timestamp: timestamp );
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
      return (apiClient.deserialize(_decodeBodyBytes(response), 'List<WireguardPeer>') as List).map((item) => item as WireguardPeer).toList();
    } else {
      return null;
    }
  }

  /// Retrieve VPN profile with HTTP info returned
  ///
  /// 
  Future<Response> getProfileWithHttpInfo() async {
    Object postBody;

    // verify required params are set

    // create path and map variables
    String path = "/wg/profile/".replaceAll("{format}","json");

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};

    List<String> contentTypes = [];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'GET',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Retrieve VPN profile
  ///
  /// 
  Future<WireguardProfile> getProfile() async {
    Response response = await getProfileWithHttpInfo();
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
      return apiClient.deserialize(_decodeBodyBytes(response), 'WireguardProfile') as WireguardProfile;
    } else {
      return null;
    }
  }

  /// Retrieve WireGuarg profile with HTTP info returned
  ///
  /// 
  Future<Response> getProfileByIDWithHttpInfo(String profileId) async {
    Object postBody;

    // verify required params are set
    if(profileId == null) {
     throw ApiException(400, "Missing required param: profileId");
    }

    // create path and map variables
    String path = "/wg/profiles/{profileId}/".replaceAll("{format}","json").replaceAll("{" + "profileId" + "}", profileId.toString());

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};

    List<String> contentTypes = [];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'GET',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Retrieve WireGuarg profile
  ///
  /// 
  Future<WireguardProfile> getProfileByID(String profileId) async {
    Response response = await getProfileByIDWithHttpInfo(profileId);
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
      return apiClient.deserialize(_decodeBodyBytes(response), 'WireguardProfile') as WireguardProfile;
    } else {
      return null;
    }
  }

  /// Retrieve list of WireGuard profiles with HTTP info returned
  ///
  /// 
  Future<Response> getProfilesWithHttpInfo() async {
    Object postBody;

    // verify required params are set

    // create path and map variables
    String path = "/wg/profiles/".replaceAll("{format}","json");

    // query params
    List<QueryParam> queryParams = [];
    Map<String, String> headerParams = {};
    Map<String, String> formParams = {};

    List<String> contentTypes = [];

    String contentType = contentTypes.isNotEmpty ? contentTypes[0] : "application/json";
    List<String> authNames = ["Bearer"];

    if(contentType.startsWith("multipart/form-data")) {
      bool hasFields = false;
      MultipartRequest mp = MultipartRequest(null, null);
      if(hasFields)
        postBody = mp;
    }
    else {
    }

    var response = await apiClient.invokeAPI(path,
                                             'GET',
                                             queryParams,
                                             postBody,
                                             headerParams,
                                             formParams,
                                             contentType,
                                             authNames);
    return response;
  }

  /// Retrieve list of WireGuard profiles
  ///
  /// 
  Future<List<WireguardProfile>> getProfiles() async {
    Response response = await getProfilesWithHttpInfo();
    if(response.statusCode >= 400) {
      throw ApiException(response.statusCode, _decodeBodyBytes(response));
    } else if(response.body != null) {
      return (apiClient.deserialize(_decodeBodyBytes(response), 'List<WireguardProfile>') as List).map((item) => item as WireguardProfile).toList();
    } else {
      return null;
    }
  }

}
