part of forestvpn_api.api;

class AppleStorePendingRenewalInfo {
  /* The current renewal preference for the auto-renewable subscription. The value for this key corresponds to the productIdentifier property of the product that the customer’s subscription renews. This field is only present if the user downgrades or crossgrades to a subscription of a different duration for the subsequent subscription period. */
  String autoRenewProductId = null;
  /* The current renewal status for the auto-renewable subscription. See https://developer.apple.com/documentation/appstorereceipts/auto_renew_status for more information. */
  String autoRenewStatus = null;
  /* The reason a subscription expired. This field is only present for a receipt that contains an expired auto-renewable subscription. https://developer.apple.com/documentation/appstorereceipts/expiration_intent Possible Values  1: The customer voluntarily canceled their subscription. 2: Billing error; for example, the customer's payment information was no longer valid. 3: The customer did not agree to a recent price increase. 4: The product was not available for purchase at the time of renewal. 5: Unknown error.  */
  String expirationIntent = null;
  //enum expirationIntentEnum {  1,  2,  3,  4,  5,  };{
  /* The time at which the grace period for subscription renewals expires, in a date-time format similar to the ISO 8601. */
  DateTime gracePeriodExpiresDate = null;
  /* A flag that indicates Apple is attempting to renew an expired subscription automatically. This field is only present if an auto-renewable subscription is in the billing retry state. See https://developer.apple.com/documentation/appstorereceipts/is_in_billing_retry_period for more information. */
  String isInBillingRetryPeriod = null;
  /* The transaction identifier of the original purchase. */
  String originalTransactionId = null;
  /* The price consent status for a subscription price increase. This field is only present if the customer was notified of the price increase. The default value is \"0\" and changes to \"1\" if the customer consents. */
  String priceConsentStatus = null;
  //enum priceConsentStatusEnum {  0,  1,  };{
  /* The unique identifier of the product purchased. You provide this value when creating the product in App Store Connect, and it corresponds to the productIdentifier property of the SKPayment object stored in the transaction's payment property. */
  String productId = null;
  AppleStorePendingRenewalInfo();

  @override
  String toString() {
    return 'AppleStorePendingRenewalInfo[autoRenewProductId=$autoRenewProductId, autoRenewStatus=$autoRenewStatus, expirationIntent=$expirationIntent, gracePeriodExpiresDate=$gracePeriodExpiresDate, isInBillingRetryPeriod=$isInBillingRetryPeriod, originalTransactionId=$originalTransactionId, priceConsentStatus=$priceConsentStatus, productId=$productId, ]';
  }

  AppleStorePendingRenewalInfo.fromJson(Map<String, dynamic> json) {
    if (json == null) return;
    autoRenewProductId = json['auto_renew_product_id'];
    autoRenewStatus = json['auto_renew_status'];
    expirationIntent = json['expiration_intent'];
    gracePeriodExpiresDate = (json['grace_period_expires_date'] == null) ?
      null :
      DateTime.parse(json['grace_period_expires_date']);
    isInBillingRetryPeriod = json['is_in_billing_retry_period'];
    originalTransactionId = json['original_transaction_id'];
    priceConsentStatus = json['price_consent_status'];
    productId = json['product_id'];
  }

  Map<String, dynamic> toJson() {
    Map <String, dynamic> json = {};
    if (autoRenewProductId != null)
      json['auto_renew_product_id'] = autoRenewProductId;
    if (autoRenewStatus != null)
      json['auto_renew_status'] = autoRenewStatus;
    if (expirationIntent != null)
      json['expiration_intent'] = expirationIntent;
    if (gracePeriodExpiresDate != null)
      json['grace_period_expires_date'] = gracePeriodExpiresDate == null ? null : gracePeriodExpiresDate.toUtc().toIso8601String();
    if (isInBillingRetryPeriod != null)
      json['is_in_billing_retry_period'] = isInBillingRetryPeriod;
    if (originalTransactionId != null)
      json['original_transaction_id'] = originalTransactionId;
    if (priceConsentStatus != null)
      json['price_consent_status'] = priceConsentStatus;
    if (productId != null)
      json['product_id'] = productId;
    return json;
  }

  static List<AppleStorePendingRenewalInfo> listFromJson(List<dynamic> json) {
    return json == null ? List<AppleStorePendingRenewalInfo>() : json.map((value) => AppleStorePendingRenewalInfo.fromJson(value)).toList();
  }

  static Map<String, AppleStorePendingRenewalInfo> mapFromJson(Map<String, dynamic> json) {
    var map = Map<String, AppleStorePendingRenewalInfo>();
    if (json != null && json.isNotEmpty) {
      json.forEach((String key, dynamic value) => map[key] = AppleStorePendingRenewalInfo.fromJson(value));
    }
    return map;
  }

  // maps a json object with a list of AppleStorePendingRenewalInfo-objects as value to a dart map
  static Map<String, List<AppleStorePendingRenewalInfo>> mapListFromJson(Map<String, dynamic> json) {
    var map = Map<String, List<AppleStorePendingRenewalInfo>>();
     if (json != null && json.isNotEmpty) {
       json.forEach((String key, dynamic value) {
         map[key] = AppleStorePendingRenewalInfo.listFromJson(value);
       });
     }
     return map;
  }
}

