part of forestvpn_api.api;

class AppleStoreServerNotification {
  /* An identifier that App Store Connect generates and the App Store uses to uniquely identify the auto-renewable subscription that the user's subscription renews. Treat this value as a 64-bit integer. */
  String autoRenewAdamId = null;
  /* The product identifier of the auto-renewable subscription that the user's subscription renews. */
  String autoRenewProductId = null;
  /* The current renewal status for an auto-renewable subscription product. Note that these values are different from those of the auto_renew_status in the receipt. */
  String autoRenewStatus = null;
  //enum autoRenewStatusEnum {  true,  false,  };{
  /* The time at which the renewal status for an auto-renewable subscription was turned on or off, in a date-time format similar to the ISO 8601 standard. */
  DateTime autoRenewStatusChangeDate = null;
  /* The environment for which the receipt was generated. */
  String environment = null;
  //enum environmentEnum {  Sandbox,  PROD,  };{
  /* The reason a subscription expired. This field is only present for an expired auto-renewable subscription. See expiration_intent for more information. https://developer.apple.com/documentation/appstorereceipts/expiration_intent Possible Values  1: The customer voluntarily canceled their subscription. 2: Billing error; for example, the customer's payment information was no longer valid. 3: The customer did not agree to a recent price increase. 4: The product was not available for purchase at the time of renewal. 5: Unknown error.  */
  int expirationIntent = null;
  /* The latest Base64-encoded transaction receipt. This field appears in the notification instead of latest_receipt for expired transactions. */
  String latestExpiredReceipt = null;
  
  List<Object> latestExpiredReceiptInfo = [];
  /* The latest Base64-encoded transaction receipt. */
  String latestReceipt = null;
  /* The JSON representation of the value in latest_receipt. Note that this field is an array in the receipt but a single object in server-to-server notifications. */
  List<AppleStoreLatestReceiptInfo> latestReceiptInfo = [];
  /* The subscription event that triggered the notification. https://developer.apple.com/documentation/appstoreservernotifications/notification_type CANCEL: Indicates that either Apple customer support canceled the subscription or the user upgraded their subscription. The cancellation_date key contains the date and time of the change. DID_CHANGE_RENEWAL_PREF: Indicates the customer made a change in their subscription plan that takes effect at the next renewal. The currently active plan is not affected. DID_CHANGE_RENEWAL_STATUS: Indicates a change in the subscription renewal status. Check auto_renew_status_change_date_ms and auto_renew_status in the JSON response to know the date and time of the last status update and the current renewal status. DID_FAIL_TO_RENEW: Indicates a subscription that failed to renew due to a billing issue. Check is_in_billing_retry_period to know the current retry status of the subscription, and grace_period_expires_date to know the new service expiration date if the subscription is in a billing grace period. DID_RECOVER: Indicates successful automatic renewal of an expired subscription that failed to renew in the past. Check expires_date to determine the next renewal date and time. INITIAL_BUY: Occurs at the initial purchase of the subscription. Store latest_receipt on your server as a token to verify the user’s subscription status at any time, by validating it with the App Store. INTERACTIVE_RENEWAL: Indicates the customer renewed a subscription interactively, either by using your app’s interface, or on the App Store in the account's Subscriptions settings. Make service available immediately. RENEWAL: Indicates successful automatic renewal of an expired subscription that failed to renew in the past. Check expires_date to determine the next renewal date and time.  */
  String notificationType = null;
  //enum notificationTypeEnum {  CANCEL,  DID_CHANGE_RENEWAL_PREF,  DID_CHANGE_RENEWAL_STATUS,  DID_FAIL_TO_RENEW,  DID_RECOVER,  INITIAL_BUY,  INTERACTIVE_RENEWAL,  RENEWAL,  };{
  /* The same value as the shared secret you submit in the password field of the requestBody when validating receipts. */
  String password = null;
  
  AppleStoreUnifiedReceipt unifiedReceipt = null;
  AppleStoreServerNotification();

  @override
  String toString() {
    return 'AppleStoreServerNotification[autoRenewAdamId=$autoRenewAdamId, autoRenewProductId=$autoRenewProductId, autoRenewStatus=$autoRenewStatus, autoRenewStatusChangeDate=$autoRenewStatusChangeDate, environment=$environment, expirationIntent=$expirationIntent, latestExpiredReceipt=$latestExpiredReceipt, latestExpiredReceiptInfo=$latestExpiredReceiptInfo, latestReceipt=$latestReceipt, latestReceiptInfo=$latestReceiptInfo, notificationType=$notificationType, password=$password, unifiedReceipt=$unifiedReceipt, ]';
  }

  AppleStoreServerNotification.fromJson(Map<String, dynamic> json) {
    if (json == null) return;
    autoRenewAdamId = json['auto_renew_adam_id'];
    autoRenewProductId = json['auto_renew_product_id'];
    autoRenewStatus = json['auto_renew_status'];
    autoRenewStatusChangeDate = (json['auto_renew_status_change_date'] == null) ?
      null :
      DateTime.parse(json['auto_renew_status_change_date']);
    environment = json['environment'];
    expirationIntent = json['expiration_intent'];
    latestExpiredReceipt = json['latest_expired_receipt'];
    latestExpiredReceiptInfo = (json['latest_expired_receipt_info'] == null) ?
      null :
      Object.listFromJson(json['latest_expired_receipt_info']);
    latestReceipt = json['latest_receipt'];
    latestReceiptInfo = (json['latest_receipt_info'] == null) ?
      null :
      AppleStoreLatestReceiptInfo.listFromJson(json['latest_receipt_info']);
    notificationType = json['notification_type'];
    password = json['password'];
    unifiedReceipt = (json['unified_receipt'] == null) ?
      null :
      AppleStoreUnifiedReceipt.fromJson(json['unified_receipt']);
  }

  Map<String, dynamic> toJson() {
    Map <String, dynamic> json = {};
    if (autoRenewAdamId != null)
      json['auto_renew_adam_id'] = autoRenewAdamId;
    if (autoRenewProductId != null)
      json['auto_renew_product_id'] = autoRenewProductId;
    if (autoRenewStatus != null)
      json['auto_renew_status'] = autoRenewStatus;
    if (autoRenewStatusChangeDate != null)
      json['auto_renew_status_change_date'] = autoRenewStatusChangeDate == null ? null : autoRenewStatusChangeDate.toUtc().toIso8601String();
    if (environment != null)
      json['environment'] = environment;
    if (expirationIntent != null)
      json['expiration_intent'] = expirationIntent;
    if (latestExpiredReceipt != null)
      json['latest_expired_receipt'] = latestExpiredReceipt;
    if (latestExpiredReceiptInfo != null)
      json['latest_expired_receipt_info'] = latestExpiredReceiptInfo;
    if (latestReceipt != null)
      json['latest_receipt'] = latestReceipt;
    if (latestReceiptInfo != null)
      json['latest_receipt_info'] = latestReceiptInfo;
    if (notificationType != null)
      json['notification_type'] = notificationType;
    if (password != null)
      json['password'] = password;
    if (unifiedReceipt != null)
      json['unified_receipt'] = unifiedReceipt;
    return json;
  }

  static List<AppleStoreServerNotification> listFromJson(List<dynamic> json) {
    return json == null ? List<AppleStoreServerNotification>() : json.map((value) => AppleStoreServerNotification.fromJson(value)).toList();
  }

  static Map<String, AppleStoreServerNotification> mapFromJson(Map<String, dynamic> json) {
    var map = Map<String, AppleStoreServerNotification>();
    if (json != null && json.isNotEmpty) {
      json.forEach((String key, dynamic value) => map[key] = AppleStoreServerNotification.fromJson(value));
    }
    return map;
  }

  // maps a json object with a list of AppleStoreServerNotification-objects as value to a dart map
  static Map<String, List<AppleStoreServerNotification>> mapListFromJson(Map<String, dynamic> json) {
    var map = Map<String, List<AppleStoreServerNotification>>();
     if (json != null && json.isNotEmpty) {
       json.forEach((String key, dynamic value) {
         map[key] = AppleStoreServerNotification.listFromJson(value);
       });
     }
     return map;
  }
}

